# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: reflection_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18reflection_service.proto\x12\x0bsumma.proto\"K\n\x12GetTopTermsRequest\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nfield_name\x18\x02 \x01(\t\x12\r\n\x05top_k\x18\x03 \x01(\r\"\xaa\x01\n\x13GetTopTermsResponse\x12\x45\n\x0bper_segment\x18\x01 \x03(\x0b\x32\x30.summa.proto.GetTopTermsResponse.PerSegmentEntry\x1aL\n\x0fPerSegmentEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.summa.proto.SegmentTerms:\x02\x38\x01\"9\n\x0cSegmentTerms\x12)\n\nterm_infos\x18\x01 \x03(\x0b\x32\x15.summa.proto.TermInfo\")\n\x08TermInfo\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x10\n\x08\x64oc_freq\x18\x02 \x01(\r2e\n\rReflectionApi\x12T\n\rget_top_terms\x12\x1f.summa.proto.GetTopTermsRequest\x1a .summa.proto.GetTopTermsResponse\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'reflection_service_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _GETTOPTERMSRESPONSE_PERSEGMENTENTRY._options = None
  _GETTOPTERMSRESPONSE_PERSEGMENTENTRY._serialized_options = b'8\001'
  _GETTOPTERMSREQUEST._serialized_start=41
  _GETTOPTERMSREQUEST._serialized_end=116
  _GETTOPTERMSRESPONSE._serialized_start=119
  _GETTOPTERMSRESPONSE._serialized_end=289
  _GETTOPTERMSRESPONSE_PERSEGMENTENTRY._serialized_start=213
  _GETTOPTERMSRESPONSE_PERSEGMENTENTRY._serialized_end=289
  _SEGMENTTERMS._serialized_start=291
  _SEGMENTTERMS._serialized_end=348
  _TERMINFO._serialized_start=350
  _TERMINFO._serialized_end=391
  _REFLECTIONAPI._serialized_start=393
  _REFLECTIONAPI._serialized_end=494
# @@protoc_insertion_point(module_scope)
